<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

namespace smsgateway_messagebird\local\service;

use core_sms\message_status;
use smsgateway_messagebird\helper;
use smsgateway_messagebird\local\messagebird_sms_service_provider;
use stdClass;

/**
 * messagebird SNS service provider.
 *
 * @package    smsgateway_messagebird
 * @copyright  2024 RvD <helpdesk@sebsoft.nl>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class messagebird_sms implements messagebird_sms_service_provider {

    /**
     * Get API Client
     *
     * @param stdClass $config
     * @return \MessageBird\Client
     */
    private static function get_api_client($config) {
        $instance = new \MessageBird\Client($config->access_key);
        return $instance;
    }

    /**
     * Include the required calls.
     */
    private static function require(): void {
        global $CFG;
        require_once($CFG->dirroot . '/sms/gateway/messagebird/vendor/messagebird/autoload.php');
    }

    #[\Override]
    public static function send_sms_message(
        string $messagecontent,
        string $phonenumber,
        stdclass $config,
    ): message_status {
        global $SITE;
        self::require();

        try {
            // Set up the sender information.
            $senderid = $SITE->shortname;
            // Remove spaces and non-alphanumeric characters from ID.
            $senderid = preg_replace("/[^A-Za-z0-9]/", '', trim($senderid));
            // We have to truncate the senderID to 11 chars.
            $senderid = substr($senderid, 0, 11);

            $message = new \MessageBird\Objects\Message();
            $message->originator = $senderid;
            $message->recipients = [$phonenumber];
            $message->body = $messagecontent;
            $message->datacoding = \MessageBird\Objects\Message::DATACODING_PLAIN;
            $message->direction = 'mt';
            $message->setFlash(false);
            $message->type = \MessageBird\Objects\Message::TYPE_SMS;

            $mb = self::get_api_client($config);
            $result = $mb->messages->create($message);

            // Since we only send to one single recipient we can fetch the first...
            // ... (and only) one and use it's status.
            $recipient = reset($result->recipients);
            switch ($recipient->status) {
                case 'buffered':
                case 'scheduled':
                    $status = \core_sms\message_status::GATEWAY_QUEUED;
                    break;
                case 'delivered':
                case 'sent':
                    $status = \core_sms\message_status::GATEWAY_SENT;
                    break;
                case 'delivery_failed':
                    $status = \core_sms\message_status::GATEWAY_FAILED;
                    break;
            }

            return $status;
        } catch (MessageBird\Exceptions\AuthenticateException $aex) {
            return \core_sms\message_status::GATEWAY_REJECTED;
        } catch (MessageBird\Exceptions\BalanceException $bex) {
            return \core_sms\message_status::GATEWAY_REJECTED;
        } catch (\Exception $e) {
            return \core_sms\message_status::GATEWAY_NOT_AVAILABLE;
        }
    }
}

